/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
*
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.wtp.releng.tools.component.api;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.eclipse.wtp.releng.tools.component.model.Type;

public class ClassAPI extends Type
{
  private String superClass;
  private int access;
  private Map methodAPIs;
  private Map fieldAPIs;
  private JavadocCoverage javadocCoverage;

  public ClassAPI()
  {
    access = -1;
  }

  public String getSuperClass()
  {
    return superClass;
  }

  public void setSuperClass(String superClass)
  {
    this.superClass = superClass;
  }

  /**
   * @return Returns the access.
   */
  public int getAccess()
  {
    return access;
  }

  /**
   * @param access The access to set.
   */
  public void setAccess(int access)
  {
    this.access = access;
  }

  /**
   * @return Returns the fieldAPIs.
   */
  public Collection getFieldAPIs()
  {
    if (fieldAPIs == null)
      fieldAPIs = new HashMap(1);
    return fieldAPIs.values();
  }

  public FieldAPI getFieldAPI(String name)
  {
    if (fieldAPIs == null)
      return null;
    else
      return (FieldAPI)fieldAPIs.get(name);
  }

  public void addFieldAPI(FieldAPI fieldAPI)
  {
    if (fieldAPIs == null)
      fieldAPIs = new HashMap(1);
    fieldAPIs.put(fieldAPI.getName(), fieldAPI);
  }

  public void removeFieldAPI(FieldAPI fieldAPI)
  {
    fieldAPIs.remove(fieldAPI.getName());
  }

  public int sizeFieldAPIs()
  {
    return fieldAPIs != null ? fieldAPIs.size() : 0;
  }

  /**
   * @return Returns the methodAPIs.
   */
  public Collection getMethodAPIs()
  {
    if (methodAPIs == null)
      methodAPIs = new HashMap(1);
    return methodAPIs.values();
  }

  public MethodAPI getMethodAPI(String name, String descriptor)
  {
    if (methodAPIs == null)
      return null;
    else
    {
      StringBuffer key = new StringBuffer();
      key.append(name);
      key.append(" ");
      key.append(descriptor);
      return (MethodAPI)methodAPIs.get(key.toString());
    }
  }

  public void addMethodAPI(MethodAPI methodAPI)
  {
    if (methodAPIs == null)
      methodAPIs = new HashMap(1);
    StringBuffer key = new StringBuffer();
    key.append(methodAPI.getName());
    key.append(" ");
    key.append(methodAPI.getDescriptor());
    methodAPIs.put(key.toString(), methodAPI);
  }

  public void removeMethodAPI(MethodAPI methodAPI)
  {
    StringBuffer key = new StringBuffer();
    key.append(methodAPI.getName());
    key.append(" ");
    key.append(methodAPI.getDescriptor());
    methodAPIs.remove(key.toString());
  }

  public int sizeMethodAPIs()
  {
    return methodAPIs != null ? methodAPIs.size() : 0;
  }

  public JavadocCoverage getJavadocCoverage()
  {
    if (javadocCoverage == null)
      javadocCoverage = new JavadocCoverage();
    return javadocCoverage;
  }

  public void setJavadocCoverage(JavadocCoverage javadocCoverage)
  {
    this.javadocCoverage = javadocCoverage;
  }

  public boolean isSetJavadocCoverage()
  {
    return javadocCoverage != null;
  }

  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("<class-api");
    sb.append(toAttribute("name", getName()));
    sb.append(toAttribute("super", getSuperClass()));
    int access = getAccess();
    if (access != -1)
      sb.append(toAttribute("access", String.valueOf(access)));
    if (getReference() != null)
      sb.append(toAttribute("reference", String.valueOf(isReference())));
    if (getImplement() != null)
      sb.append(toAttribute("implement", String.valueOf(isImplement())));
    if (getSubclass() != null)
      sb.append(toAttribute("subclass", String.valueOf(isSubclass())));
    if (getInstantiate() != null)
      sb.append(toAttribute("instantiate", String.valueOf(isInstantiate())));
    sb.append(">");
    if (javadocCoverage != null)
      sb.append(javadocCoverage.toString());
    if (sizeMethodAPIs() > 0)
      for (Iterator it = getMethodAPIs().iterator(); it.hasNext();)
        sb.append(((MethodAPI)it.next()).toString());
    if (sizeFieldAPIs() > 0)
      for (Iterator it = getFieldAPIs().iterator(); it.hasNext();)
        sb.append(((FieldAPI)it.next()).toString());
    sb.append("</class-api>");
    return sb.toString();
  }
}