/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.wtp.releng.tools.component.dependency;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.core.runtime.IPlatformRunnable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.wtp.releng.tools.component.images.ImagesUtil;

public class UIDependencyScanner implements IPlatformRunnable
{
  public Object run(Object arguments)
  {
    String outputDir = addTrailingSeperator(System.getProperty("outputDir"));
    PluginDependencyScanner scanner = new PluginDependencyScanner();
    Map plugins = scanner.execute();
    StringBuffer content = new StringBuffer();
    content.append("<html>");
    content.append("<script language=\"javascript\">");
    content.append("function twist(img)");
    content.append("{");
    content.append("var div = document.getElementById(img.id.substring(0, img.id.length - 4));");
    content.append("if (div.style.display == \"\")");
    content.append("{");
    content.append("div.style.display = \"none\";");
    content.append("img.src = \"twistclosed.gif\";");
    content.append("}");
    content.append("else");
    content.append("{");
    content.append("div.style.display = \"\";");
    content.append("img.src = \"twistopened.gif\";");
    content.append("}");
    content.append("}");
    content.append("</script>");
    content.append("<body>");
    content.append("<table border=\"0\" cellpadding=\"2\" cellspacing=\"5\" width=\"100%\">");
    content.append("<tr>");
    content.append("<td align=\"left\" width=\"60%\">");
    content.append("<font style=\"font-size: x-large;; font-family: Verdana, Arial, Helvetica, sans-serif; font-weight: bold\">UI Dependency Report</font>");
    content.append("</td>");
    content.append("<td width=\"40%\">");
    content.append("<img src=\"Idea.jpg\" align=\"middle\" height=\"86\" hspace=\"50\" width=\"120\"/>");
    content.append("</td>");
    content.append("</tr>");
    content.append("</table>");
    content.append("<table border=\"0\" cellpadding=\"2\" cellspacing=\"5\" width=\"100%\">");
    content.append("<tr>");
    content.append("<td ALIGN=\"LEFT\" VALIGN=\"TOP\" COLSPAN=\"2\" BGCOLOR=\"#0080C0\"><b><font color=\"#FFFFFF\" face=\"Arial,Helvetica\">UI Dependencies</font></b></td>");
    content.append("</tr>");
    content.append("</table>");
    content.append("<font face=\"Arial,Helvetica\">");
    drawTree((Plugin)plugins.get("org.eclipse.swt"), 0, new ArrayList(), content);
    content.append("</font>");
    content.append("</body></html>");
    try
    {
      ImagesUtil.copyAllFromBundle(Platform.getBundle("org.eclipse.wtp.releng.tools.component.core"), outputDir);
      FileOutputStream fos = new FileOutputStream(outputDir + "ui-dep.html");
      byte[] b = content.toString().getBytes();
      fos.write(b);
    }
    catch (IOException ioe)
    {
      ioe.printStackTrace();
    }
    return IPlatformRunnable.EXIT_OK;
  }

  private void drawTree(Plugin plugin, int depth, List drawnPlugins, StringBuffer content)
  {
    List dependents = plugin.getDependents();
    String id = plugin.getId();
    if (!drawnPlugins.contains(id))
    {
      drawnPlugins.add(id);
      for (int i = 0; i < depth - 1; i++)
        content.append("<img border=0 src=\"line.gif\"/>");
      if (depth > 0)
      {
        content.append("<img border=0 src=\"line_notlast.gif\"/>");
        /*
        if (dependents.isEmpty())
          content.append("<img border=0 src=\"space.gif\"/>");
        else
        {
          content.append("<img border=0 src=\"twistopened.gif\" id=\"");
          content.append(id);
          content.append(".img");
          content.append("\" onclick=\"javascript:twist(this)\"/>");
        }
        */
      }
      content.append(id);
      content.append("<br>");
    }
    if (!dependents.isEmpty())
    {
      /*
      content.append("<div id=\"");
      content.append(id);
      content.append("\">");
      */
      for (Iterator it = dependents.iterator(); it.hasNext();)
        drawTree((Plugin)it.next(), depth + 1, drawnPlugins, content);
      //content.append("</div>");
    }
  }

  protected String addTrailingSeperator(String s)
  {
    if (s != null && !s.endsWith("/") && !s.endsWith("\\"))
    {
      StringBuffer sb = new StringBuffer(s);
      sb.append('/');
      return sb.toString();
    }
    else
    {
      return s;
    }
  }
}